/* $Id: ui_dc.c,v 1.12 2003/01/11 00:17:11 chris Exp $
 *
 * Dreamcast port UI driver
 * (c) 2002,2003 Christian Groessler
 *
 * derived from wince/port/ui_wince.c by Vasyl Tsvirkunov
 */

#include <string.h>
#include <malloc.h>
#include "atari.h"
#include "ui.h"
#include "input.h"
#include "version.h"

extern int emulate_paddles;
extern int glob_snd_ena;
extern int db_mode;
extern int screen_tv_mode;
extern int in_fnedit;
extern void update_vidmode(void);
extern void update_screen_updater(void);
extern int x_ovr, y_ovr, b_ovr;
extern int x_key, y_key, b_key;
#ifdef HZ_TEST
extern void do_hz_test(void);
#endif
extern int get_emkey(UBYTE *);

static int DCUISelect(char* pTitle, int bFloat, int nDefault, tMenuItem* menu, int* ascii);
static int DCUIGetSaveFilename(char* pFilename);
static int DCUIGetLoadFilename(char* pDirectory, char* pFilename);
static void DCUIMessage(char* pMessage);
static void DCUIAboutBox();
static void DCUIInit();


tUIDriver dc_ui_driver =
{
	&DCUISelect,
	&DCUIGetSaveFilename,
	&DCUIGetLoadFilename,
	&DCUIMessage,
	&DCUIAboutBox,
	&DCUIInit
};

static void AboutAtariDC()
{
        UBYTE *screen = (UBYTE*)atari_screen;

        ClearScreen(screen);
        Box(screen, 0x9a, 0x94, 0, 0, 39, 23);

        CenterPrint(screen, 0x9a, 0x94, "AtariDC v" A800DCVERASC " ("__DATE__")", 2);
        CenterPrint(screen, 0x9a, 0x94, "(c) 2002,2003 Christian Groessler", 4);
        CenterPrint(screen, 0x9a, 0x94, "http://www.groessler.org/a800dc", 5);

        CenterPrint(screen, 0x9a, 0x94, "Please report all problems", 7);
        CenterPrint(screen, 0x9a, 0x94, "to chris@groessler.org", 8);

        CenterPrint(screen, 0x9a, 0x94, "This port is based on", 10);
        CenterPrint(screen, 0x9a, 0x94, ATARI_TITLE, 11);
        CenterPrint(screen, 0x9a, 0x94, "http://atari800.atari.org", 12);

        CenterPrint(screen, 0x9a, 0x94, "It uses the KallistiOS library", 14);
        CenterPrint(screen, 0x9a, 0x94, "http://cadcdev.sourceforge.net", 15);

        CenterPrint(screen, 0x94, 0x9a, "Press any Key to Continue", 22);
        GetKeyPress(screen);
}


/* provide "centered" names */
static struct repl_names {
	char *sig;
	char *new_name;
} repl_names[] = {
	{ "DISK", "            Disk Management" },
	{ "CART", "          Cartridge Management" },
	{ "XBIN", "         Run BIN file directly" },
	{ "CASS", "            Load tape image" },
	{ "SYST", "             Select System" },
	{ "STER", "           Sound Mono/Stereo" },
	{ "LOAD", "              Load State" },
	{ "SAVE", "              Save State" },
	{ "ARTF", "           Artifacting mode" },
	{ "SETT", "            Atari Settings" },
	{ "CONT", "        Back to emulated Atari" },
	{ "REST", "          Reset  (Warm Start)" },
	{ "REBT", "          Reboot (Cold Start)" },
	{ "ABOU", "           About the Emulator" },
	{ "MENU", "                 Menu" },
	{ "EXIT", "             Exit Emulator" }
};
#define num_repl_names (sizeof(repl_names) / sizeof(struct repl_names))

static void EmulatorSettings()
{
	static tMenuItem menu_array[] =
	{
		{ "PADD", ITEM_ENABLED|ITEM_CHECK, NULL, "Emulate paddles:",	       NULL, 0 },
		{ "AFRE", ITEM_ENABLED|ITEM_CHECK, NULL, "Joystick autofire:",	       NULL, 1 },
		{ "SNDE", ITEM_ENABLED|ITEM_CHECK, NULL, "Enable sound:",	       NULL, 2 },
		{ "DBLB", ITEM_ENABLED|ITEM_CHECK, NULL, "Double buffer video data: ", NULL, 3 },
		{ "NTSC", ITEM_ENABLED, NULL,            "Emulator video mode:",       NULL, 4 },
		{ "NTSD", ITEM_ENABLED, NULL,            "Display video mode:",	       NULL, 5 },
#ifdef HZ_TEST
		{ "HZTS", ITEM_ENABLED|ITEM_CHECK, NULL, "DO HZ TEST:",		       NULL, 6 },
#endif
		MENU_END
	};

	int option = 0;

	do {
		if (emulate_paddles)
			menu_array[0].flags |= ITEM_CHECKED;
		else
			menu_array[0].flags &= ~ITEM_CHECKED;

		if (joy_autofire[0])
			menu_array[1].flags |= ITEM_CHECKED;
		else
			menu_array[1].flags &= ~ITEM_CHECKED;

		if (glob_snd_ena)
			menu_array[2].flags |= ITEM_CHECKED;
		else
			menu_array[2].flags &= ~ITEM_CHECKED;

		if (db_mode)
			menu_array[3].flags |= ITEM_CHECKED;
		else
			menu_array[3].flags &= ~ITEM_CHECKED;

		if (tv_mode == TV_NTSC) {
			menu_array[4].suffix = "NTSC";
		}
		else {
			menu_array[4].suffix = "PAL";
		}

		if (screen_tv_mode == TV_NTSC) {
			menu_array[5].suffix = "NTSC";
		}
		else {
			menu_array[5].suffix = "PAL";
		}

#ifdef HZ_TEST
		menu_array[6].flags &= ~ITEM_CHECKED;
#endif
		option = basic_ui_driver.fSelect(NULL, TRUE, option, menu_array, NULL);

		switch(option) {
		case 0:
			emulate_paddles = !emulate_paddles;
			break;
		case 1:
			joy_autofire[0] = joy_autofire[0] ? 0 : 1;
			break;
		case 2:
			glob_snd_ena = !glob_snd_ena;
			break;
		case 3:
			if (db_mode) {
				db_mode = FALSE;
			}
			else if (tv_mode == screen_tv_mode) {
				db_mode = TRUE;
			}
			update_screen_updater();
			entire_screen_dirty();
			break;
		case 4:
			if (tv_mode == TV_NTSC) {
				tv_mode = TV_PAL;
			}
			else {
				tv_mode = TV_NTSC;
			}
			if (tv_mode != screen_tv_mode) {
				db_mode = FALSE;
				update_screen_updater();
			}
			update_vidmode();
			entire_screen_dirty();
			break;
		case 5:
			if (screen_tv_mode == TV_NTSC) {
				screen_tv_mode = TV_PAL;
				tv_mode = TV_PAL;
			}
			else {
				screen_tv_mode = TV_NTSC;
				tv_mode = TV_NTSC;
			}
			update_vidmode();
			entire_screen_dirty();
			break;
#ifdef HZ_TEST
		case 6:
			do_hz_test();
			entire_screen_dirty();
			break;
#endif
		}
	} while(option >= 0);
}

extern char key_to_ascii[];
static void ovr2str(int keycode, char *keystr)
{
	sprintf(keystr, "\"%c\" ($%02X)", key_to_ascii[keycode], keycode);
}

static void ControllerSettings()
{
	static tMenuItem menu_array[] =
	{
		{ "CTSX", ITEM_ENABLED, NULL, "Override X key: ", NULL, 0 },
		{ "CTSY", ITEM_ENABLED, NULL, "Override Y key: ", NULL, 1 },
		{ "CTSB", ITEM_ENABLED, NULL, "Override B key: ", NULL, 2 },
		MENU_END
	};

	char keystr[3][10];

	int option = 0;
	do {
		if (x_ovr) {
			ovr2str(x_key, keystr[0]);
			menu_array[0].suffix = keystr[0];
		}
		else {
			menu_array[0].suffix = "OFF      ";
		}

		if (y_ovr) {
			ovr2str(y_key, keystr[1]);
			menu_array[1].suffix = keystr[1];
		}
		else {
			menu_array[1].suffix = "OFF      ";
		}

		if (b_ovr) {
			ovr2str(b_key, keystr[2]);
			menu_array[2].suffix = keystr[2];
		}
		else {
			menu_array[2].suffix = "OFF      ";
		}

		/*option = ui_driver->fSelect(NULL, TRUE, option, menu_array, NULL);*/
		/*option = BasicUISelect(NULL, TRUE, option, menu_array, NULL);*/
		option = basic_ui_driver.fSelect(NULL, TRUE, option, menu_array, NULL);

		switch(option) {
		case 0:
			if (x_ovr) x_ovr = FALSE;
			else {
				x_key = get_emkey("Select X button definition");
				if (x_key != AKEY_NONE) {
					x_ovr = TRUE;
				}
				else {
					x_ovr = FALSE;
				}
			}
			break;
		case 1:
			if (y_ovr) y_ovr = FALSE;
			else {
				y_key = get_emkey("Select Y button definition");
				if (y_key != AKEY_NONE) {
					y_ovr = TRUE;
				}
				else {
					y_ovr = FALSE;
				}
			}
			break;
		case 2:
			if (b_ovr) b_ovr = FALSE;
			else {
				b_key = get_emkey("Select B button definition");
				if (b_key != AKEY_NONE) {
					b_ovr = TRUE;
				}
				else {
					b_ovr = FALSE;
				}
			}
			break;
		}
	} while(option >= 0);
}


static int DCUISelect(char* pTitle, int bFloat, int nDefault, tMenuItem* menu, int* ascii)
{
	tMenuItem* pNewMenu;
	int i, j, k;
	int result;

	/* Count items in old menu */
	for(i=0; menu[i].sig; i++)
		;

	/* Allocate memory for new menu, with space for extra items */
	pNewMenu = (tMenuItem *)malloc(sizeof(tMenuItem) * (i + 3));

	/* Copy old menu items, modify them appropriately, insert some new items */
	for (i=0, k=0; menu[k].sig; /*i++,*/ k++) {
#ifndef STEREO
		if (strcmp(menu[k].sig, "STER") == 0)
			continue;	/* Stereo is disabled based on compile settings */
#endif
		if (strcmp(menu[k].sig, "SREC") == 0)
			continue;	/* Sound recording is disabled */
		if (strcmp(menu[k].sig, "PCXN") == 0)
			continue;
		if (strcmp(menu[k].sig, "MONI") == 0)
			continue;

		pNewMenu[i] = menu[k];
		pNewMenu[i].suffix = NULL; /* no keyboard shortcuts in AtariDC */

		/* modify name (center it) */
		for (j=0; j<num_repl_names; j++)
			if (strcmp(pNewMenu[i].sig, repl_names[j].sig) == 0) {
				pNewMenu[i].item = repl_names[j].new_name;
				break;
			}

		/* insert AtariDC settings after main settings */
		if (strcmp(pNewMenu[i].sig, "SETT") == 0) {
			pNewMenu[++i] = menu[k];
			pNewMenu[i].sig = "EMUX";
			pNewMenu[i].flags = ITEM_ENABLED | ITEM_SUBMENU;
			pNewMenu[i].prefix = NULL;
			pNewMenu[i].item = "           AtariDC Settings";
			pNewMenu[i].suffix = NULL;
			pNewMenu[i].retval = 1000;
			pNewMenu[++i] = menu[k];
			pNewMenu[i].sig = "EMUK";
			pNewMenu[i].flags = ITEM_ENABLED | ITEM_SUBMENU;
			pNewMenu[i].prefix = NULL;
			pNewMenu[i].item = "         Controller  Settings";
			pNewMenu[i].suffix = NULL;
			pNewMenu[i].retval = 1002;
		}
		/* insert AtariDC about box after main about box */
		else if (strcmp(pNewMenu[i].sig, "ABOU") == 0) {
			pNewMenu[++i] = menu[k];
			pNewMenu[i].sig = "ABDC";
			pNewMenu[i].flags = ITEM_ENABLED | ITEM_SUBMENU;
			pNewMenu[i].prefix = NULL;
			pNewMenu[i].item = "             About AtariDC";
			pNewMenu[i].suffix = NULL;
			pNewMenu[i].retval = 1001;
		}
		i++;
	}
	pNewMenu[i] = menu[k];  /* add empty menu entry as end marker */

	/* feed it to the basic driver */
	result = basic_ui_driver.fSelect(pTitle, bFloat, nDefault, pNewMenu, ascii);

	/* Free memory */
	free((void*)pNewMenu);

	/* done */
	switch(result) {
	case 1000:
		EmulatorSettings();
		break;
	case 1001:
		AboutAtariDC();
		break;
	case 1002:
		ControllerSettings();
		break;
	}
	return result;
}

/* returns TRUE if valid filename */
static int DCEditFilename(UBYTE *screen, char *fname)
{
	int i;

	if (strlen(fname) < FILENAME_SIZE) {
		for (i = strlen(fname); i<FILENAME_SIZE; i++) *(fname + i) = ' ';
	}
	fname[FILENAME_SIZE] = '\0';
	Box(screen, 0x9a, 0x94, 3, 9, 36, 11);
	Print(screen, 0x94, 0x9a, "Filename", 4, 9);
	if (!EditString(screen, 0x9a, 0x94, FILENAME_SIZE, fname, 4, 10))
		return FALSE;
	RemoveSpaces(fname);
	return fname[0] != '\0';
}

static int DCUIGetSaveFilename(char* pFilename)
{
	int r;
	strcpy(pFilename, "A800.SAV");	 /* default save filename */
	in_fnedit = TRUE;
	r = DCEditFilename((UBYTE*)atari_screen, pFilename);
	in_fnedit = FALSE;
}

static int DCUIGetLoadFilename(char* pDirectory, char* pFilename)
{
	return basic_ui_driver.fGetLoadFilename(pDirectory, pFilename);
}

static void DCUIMessage(char* pMessage)
{
	basic_ui_driver.fMessage(pMessage);
}

static void DCUIAboutBox()
{
	basic_ui_driver.fAboutBox();
}

static void DCUIInit()
{
	basic_ui_driver.fInit();
}

void dc_printbox(UBYTE *screen, char *string)
{
	int l = strlen(string);

	Box(screen, 0x9a, 0x94, 20-l/2-1, 11, 20-l/2-1+l+1, 13);
	Print(screen, 0x94, 0x9a, string, 20-l/2, 12);
}
